import { TCacheFunction } from './cache-function';
export type TCacheFactory = (lastNumberWeakMap: WeakMap<Map<number, any> | Set<number>, number>) => TCacheFunction;
//# sourceMappingURL=cache-factory.d.ts.map