import { EmberInitialSecurityState } from '../types/struct';
import crc16ccitt from './crc16ccitt';
declare function ember_security(networkKey: Buffer): EmberInitialSecurityState;
export { crc16ccitt, ember_security };
//# sourceMappingURL=index.d.ts.map