import { load } from 'worker-timers-broker';
export declare const clearInterval: ReturnType<typeof load>['clearInterval'];
export declare const clearTimeout: ReturnType<typeof load>['clearTimeout'];
export declare const setInterval: ReturnType<typeof load>['setInterval'];
export declare const setTimeout: ReturnType<typeof load>['setTimeout'];
//# sourceMappingURL=module.d.ts.map