import { TCacheFunction } from './cache-function';
import { TGenerateUniqueNumberFunction } from './generate-unique-number-function';
export type TGenerateUniqueNumberFactory = (cache: TCacheFunction, lastNumberWeakMap: WeakMap<Map<number, any> | Set<number>, number>) => TGenerateUniqueNumberFunction;
//# sourceMappingURL=generate-unique-number-factory.d.ts.map