import { Struct } from '../struct';
/**
 * Creates a network PAN ID struct.
 *
 * @param data Data to initialize structure with.
 */
export declare const nwkPanId: (data?: Buffer) => import("../struct").BuiltStruct<Struct & Record<"panId", number>>;
//# sourceMappingURL=nwk-pan-id.d.ts.map