/**
 * This module's only purpose is to build the models index for zigbee-herdsman-converters.
 */
export declare function buildIndex(fromSrc?: boolean): Promise<void>;
//# sourceMappingURL=indexer.d.ts.map