import{_ as x}from"./_getAllKeysIn-bf2f2df3.js";import{aG as g,aw as u,ae as l,aK as m}from"./index-a77bd902.js";var I=x,K=g,p=u,f=l,O=m;function P(r,n,o,d){if(!f(r))return r;n=K(n,r);for(var i=-1,e=n.length,v=e-1,a=r;a!=null&&++i<e;){var s=O(n[i]),t=o;if(s==="__proto__"||s==="constructor"||s==="prototype")return r;if(i!=v){var _=a[s];t=d?d(_,s,a):void 0,t===void 0&&(t=f(_)?_:p(n[i+1])?[]:{})}I(a,s,t),a=a[s]}return r}var h=P;export{h as _};
