"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
// Patch BigInt serialization which is used in e.g. Zcl payloads.
// https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/BigInt#use_within_json
// eslint-disable-next-line @typescript-eslint/no-explicit-any
BigInt.prototype.toJSON = function () {
    return this.toString();
};
//# sourceMappingURL=patchBigIntSerialization.js.map