"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.tz = void 0;
const tz = {
    setModel: {
        key: ["model"],
        convertSet: async (entity, key, value, meta) => {
            await entity.write("genBasic", { modelId: value });
            return { state: { model: value } };
        },
    },
};
exports.tz = tz;
//# sourceMappingURL=sunricher.js.map