import type { KeyValueAny, KeyValueNumberString } from "./types";
export declare const OneJanuary2000: number;
export declare const defaultBindGroup = 901;
export declare const repInterval: {
    HOUR: number;
    MAX: number;
    MINUTE: number;
    SECONDS_10: number;
    MINUTES_10: number;
    MINUTES_15: number;
    MINUTES_30: number;
    MINUTES_5: number;
    SECONDS_5: number;
};
export declare const thermostatControlSequenceOfOperations: KeyValueNumberString;
export declare const thermostatProgrammingOperationModes: KeyValueNumberString;
export declare const thermostatSystemModes: KeyValueNumberString;
export declare const acovaThermostatSystemModes: KeyValueNumberString;
export declare const thermostatRunningMode: KeyValueNumberString;
export declare const thermostatDayOfWeek: KeyValueNumberString;
export declare const thermostatRunningStates: KeyValueAny;
export declare const thermostatAcLouverPositions: KeyValueNumberString;
export declare const thermostatScheduleMode: KeyValueNumberString;
export declare const fanMode: {
    off: number;
    low: number;
    medium: number;
    high: number;
    on: number;
    auto: number;
    smart: number;
};
export declare const temperatureDisplayMode: KeyValueNumberString;
export declare const danfossAdaptionRunStatus: KeyValueNumberString;
export declare const danfossAdaptionRunControl: KeyValueNumberString;
export declare const danfossWindowOpen: KeyValueNumberString;
export declare const danfossRoomStatusCode: KeyValueNumberString;
export declare const danfossRoomFloorSensorMode: KeyValueNumberString;
export declare const danfossOutputStatus: KeyValueNumberString;
export declare const danfossSystemStatusWater: KeyValueNumberString;
export declare const danfossSystemStatusCode: KeyValueNumberString;
export declare const danfossMultimasterRole: KeyValueNumberString;
export declare const develcoInterfaceMode: KeyValueAny;
export declare const keypadLockoutMode: KeyValueAny;
export declare const lockSourceName: KeyValueNumberString;
export declare const armMode: KeyValueNumberString;
export declare const armNotification: {
    0: string;
    1: string;
    2: string;
    3: string;
    4: string;
    5: string;
    6: string;
};
export declare const ColorMode: {
    HS: number;
    XY: number;
    ColorTemp: number;
};
export declare const colorModeLookup: {
    [x: number]: string;
};
export declare const lockSoundVolume: string[];
export declare const lockUserStatus: KeyValueNumberString;
export declare const easyCodeTouchActions: KeyValueNumberString;
export declare const wiserDimmerControlMode: KeyValueNumberString;
//# sourceMappingURL=constants.d.ts.map