export * from './types/index';
declare const generateUniqueNumber: import("./types/generate-unique-number-function").TGenerateUniqueNumberFunction;
declare const addUniqueNumber: import("./types/add-unique-number-function").TAddUniqueNumberFunction;
export { addUniqueNumber, generateUniqueNumber };
//# sourceMappingURL=module.d.ts.map