import * as Zcl from '../../zspec/zcl';
import { CustomClusters } from '../../zspec/zcl/definition/tstype';
interface KeyValue {
    [s: string]: number | string;
}
declare function attributeKeyValue(frame: Zcl.Frame, deviceManufacturerID: number | undefined, customClusters: CustomClusters): KeyValue;
declare function attributeList(frame: Zcl.Frame, deviceManufacturerID: number | undefined, customClusters: CustomClusters): Array<string | number>;
export { attributeKeyValue, attributeList };
//# sourceMappingURL=zclFrameConverter.d.ts.map