import{aP as i}from"./index-a77bd902.js";const n=""+new URL("generic-zigbee-device-c9aa86d9.png",import.meta.url).href,a="https://www.zigbee2mqtt.io/images/devices/",o=e=>{var t;return`${a}${i((t=e==null?void 0:e.definition)==null?void 0:t.model)}.png`},s=e=>{var t;return(t=e.definition)==null?void 0:t.icon},m=[s,o,()=>n];export{m as A,n as a,o as g};
