export declare const definitions: {
    fingerprint: {
        modelID: string;
        manufacturerName: string;
    }[];
    model: string;
    vendor: string;
    description: string;
    extend: import("../lib/types").ModernExtend[];
    ota: boolean;
}[];
//# sourceMappingURL=zigbeetlc.d.ts.map