import { type NumericArgs } from "./modernExtend";
import type { ModernExtend } from "./types";
export declare const develcoModernExtend: {
    addCustomClusterManuSpecificDevelcoGenBasic: () => ModernExtend;
    addCustomClusterManuSpecificDevelcoAirQuality: () => ModernExtend;
    readGenBasicPrimaryVersions: () => ModernExtend;
    voc: (args?: Partial<NumericArgs>) => ModernExtend;
    airQuality: () => ModernExtend;
    batteryLowAA: () => ModernExtend;
    temperature: (args?: Partial<NumericArgs>) => ModernExtend;
    deviceTemperature: (args?: Partial<NumericArgs>) => ModernExtend;
    currentSummation: (args?: Partial<NumericArgs>) => ModernExtend;
    pulseConfiguration: (args?: Partial<NumericArgs>) => ModernExtend;
};
//# sourceMappingURL=develco.d.ts.map