import { TAddUniqueNumberFunction } from './add-unique-number-function';
import { TGenerateUniqueNumberFunction } from './generate-unique-number-function';
export type TAddUniqueNumberFactory = (generateUniqueNumber: TGenerateUniqueNumberFunction) => TAddUniqueNumberFunction;
//# sourceMappingURL=add-unique-number-factory.d.ts.map