import { Buffer } from 'node:buffer';
type calcFn = (buf: Buffer | number[], previous: number) => number;
declare const crc16ccitt: calcFn;
export default crc16ccitt;
//# sourceMappingURL=crc16ccitt.d.ts.map