/** Mapping of power source bits to descriptive string. */
export declare const POWER_SOURCES: Readonly<{
    [s: number]: string;
}>;
/** Mapping of device type to ID */
export declare const ENDPOINT_DEVICE_TYPE: Readonly<{
    [s: string]: number;
}>;
//# sourceMappingURL=consts.d.ts.map