import{j as s}from"./index-a77bd902.js";const e=()=>s.jsxs("div",{className:"d-flex align-items-center gap-2",children:[s.jsx("span",{className:"mr-2",children:"Loading, please wait."}),s.jsx("div",{className:"spinner-border ml-2"})]}),r=e;export{r as S};
