import { PacketHandler } from '../shared';
export declare const ReasonCodes: {
    0: string;
    1: string;
    2: string;
    3: string;
    4: string;
    5: string;
    16: string;
    17: string;
    128: string;
    129: string;
    130: string;
    131: string;
    132: string;
    133: string;
    134: string;
    135: string;
    136: string;
    137: string;
    138: string;
    139: string;
    140: string;
    141: string;
    142: string;
    143: string;
    144: string;
    145: string;
    146: string;
    147: string;
    148: string;
    149: string;
    150: string;
    151: string;
    152: string;
    153: string;
    154: string;
    155: string;
    156: string;
    157: string;
    158: string;
    159: string;
    160: string;
    161: string;
    162: string;
};
declare const handleAck: PacketHandler;
export default handleAck;
