._action-column_1729o_1 span{padding:0}._action-column_1729o_1 i{margin-left:5px}._device-pic_1729o_8{padding-top:0!important;padding-bottom:0!important;min-height:56px}._device-image_1729o_14 img{max-height:2em}
