import type { Tz } from "./types";
declare const tz: {
    setModel: {
        key: string[];
        convertSet: (entity: import("zigbee-herdsman/dist/controller/model").Group | import("zigbee-herdsman/dist/controller/model").Endpoint, key: string, value: unknown, meta: Tz.Meta) => Promise<{
            state: {
                model: unknown;
            };
        }>;
    };
};
export { tz };
//# sourceMappingURL=sunricher.d.ts.map