import type { Fz } from "../lib/types";
export declare const fan: Fz.Converter;
export declare const fan_speed: Fz.Converter;
export declare const thermostat: Fz.Converter;
export declare const thermostat_weekly_schedule: Fz.Converter;
export declare const hvac_user_interface: Fz.Converter;
export declare const lock_operation_event: Fz.Converter;
export declare const lock_programming_event: Fz.Converter;
export declare const lock: Fz.Converter;
export declare const lock_pin_code_response: Fz.Converter;
export declare const lock_user_status_response: Fz.Converter;
export declare const linkquality_from_basic: Fz.Converter;
export declare const battery: Fz.Converter;
export declare const temperature: Fz.Converter;
export declare const device_temperature: Fz.Converter;
export declare const humidity: Fz.Converter;
export declare const pm25: Fz.Converter;
export declare const flow: Fz.Converter;
export declare const soil_moisture: Fz.Converter;
export declare const pressure: Fz.Converter;
export declare const co2: Fz.Converter;
export declare const occupancy: Fz.Converter;
export declare const occupancy_with_timeout: Fz.Converter;
export declare const occupancy_timeout: Fz.Converter;
export declare const brightness: Fz.Converter;
export declare const level_config: Fz.Converter;
export declare const color_colortemp: Fz.Converter;
export declare const meter_identification: Fz.Converter;
export declare const metering: Fz.Converter;
export declare const electrical_measurement: Fz.Converter;
export declare const gas_metering: Fz.Converter;
export declare const on_off: Fz.Converter;
export declare const on_off_force_multiendpoint: Fz.Converter;
export declare const on_off_skip_duplicate_transaction: Fz.Converter;
export declare const power_on_behavior: Fz.Converter;
export declare const ias_no_alarm: Fz.Converter;
export declare const ias_siren: Fz.Converter;
export declare const ias_water_leak_alarm_1: Fz.Converter;
export declare const ias_water_leak_alarm_1_report: Fz.Converter;
export declare const ias_vibration_alarm_1: Fz.Converter;
export declare const ias_vibration_alarm_1_with_timeout: Fz.Converter;
export declare const ias_gas_alarm_1: Fz.Converter;
export declare const ias_gas_alarm_2: Fz.Converter;
export declare const ias_smoke_alarm_1: Fz.Converter;
export declare const ias_contact_alarm_1: Fz.Converter;
export declare const ias_contact_alarm_1_report: Fz.Converter;
export declare const ias_carbon_monoxide_alarm_1: Fz.Converter;
export declare const ias_carbon_monoxide_alarm_1_gas_alarm_2: Fz.Converter;
export declare const ias_sos_alarm_2: Fz.Converter;
export declare const ias_occupancy_alarm_1: Fz.Converter;
export declare const ias_occupancy_alarm_1_report: Fz.Converter;
export declare const ias_occupancy_alarm_2: Fz.Converter;
export declare const ias_alarm_only_alarm_1: Fz.Converter;
export declare const ias_occupancy_only_alarm_2: Fz.Converter;
export declare const ias_occupancy_alarm_1_with_timeout: Fz.Converter;
export declare const command_store: Fz.Converter;
export declare const command_recall: Fz.Converter;
export declare const command_panic: Fz.Converter;
export declare const command_arm: Fz.Converter;
export declare const command_cover_stop: Fz.Converter;
export declare const command_cover_open: Fz.Converter;
export declare const command_cover_close: Fz.Converter;
export declare const command_on: Fz.Converter;
export declare const command_off: Fz.Converter;
export declare const command_off_with_effect: Fz.Converter;
export declare const command_toggle: Fz.Converter;
export declare const command_move_to_level: Fz.Converter;
export declare const command_move: Fz.Converter;
export declare const command_step: Fz.Converter;
export declare const command_stop: Fz.Converter;
export declare const command_move_color_temperature: Fz.Converter;
export declare const command_step_color_temperature: Fz.Converter;
export declare const command_ehanced_move_to_hue_and_saturation: Fz.Converter;
export declare const command_move_to_hue_and_saturation: Fz.Converter;
export declare const command_step_hue: Fz.Converter;
export declare const command_step_saturation: Fz.Converter;
export declare const command_color_loop_set: Fz.Converter;
export declare const command_move_to_color_temp: Fz.Converter;
export declare const command_move_to_color: Fz.Converter;
export declare const command_move_hue: Fz.Converter;
export declare const command_move_to_saturation: Fz.Converter;
export declare const command_move_to_hue: Fz.Converter;
export declare const command_emergency: Fz.Converter;
export declare const command_on_state: Fz.Converter;
export declare const command_off_state: Fz.Converter;
export declare const identify: Fz.Converter;
export declare const cover_position_tilt: Fz.Converter;
export declare const cover_position_via_brightness: Fz.Converter;
export declare const cover_state_via_onoff: Fz.Converter;
export declare const curtain_position_analog_output: Fz.Converter;
export declare const lighting_ballast_configuration: Fz.Converter;
export declare const checkin_presence: Fz.Converter;
export declare const ias_enroll: Fz.Converter;
export declare const ias_wd: Fz.Converter;
export declare const power_source: Fz.Converter;
export declare const namron_thermostat: Fz.Converter;
export declare const namron_hvac_user_interface: Fz.Converter;
export declare const elko_thermostat: Fz.Converter;
export declare const ias_smoke_alarm_1_develco: Fz.Converter;
export declare const ts0201_temperature_humidity_alarm: Fz.Converter;
export declare const tuya_led_controller: Fz.Converter;
export declare const wiser_device_info: Fz.Converter;
export declare const tuya_doorbell_button: Fz.Converter;
export declare const terncy_knob: Fz.Converter;
export declare const DTB190502A1: Fz.Converter;
export declare const ZigUP: Fz.Converter;
export declare const terncy_contact: Fz.Converter;
export declare const terncy_temperature: Fz.Converter;
export declare const ts0216_siren: Fz.Converter;
export declare const tuya_cover_options_2: Fz.Converter;
export declare const tuya_cover_options: Fz.Converter;
export declare const WSZ01_on_off_action: Fz.Converter;
export declare const tuya_switch_scene: Fz.Converter;
export declare const livolo_switch_state: Fz.Converter;
export declare const livolo_socket_state: Fz.Converter;
export declare const livolo_new_switch_state: Fz.Converter;
export declare const livolo_new_switch_state_2gang: Fz.Converter;
export declare const livolo_new_switch_state_4gang: Fz.Converter;
export declare const livolo_curtain_switch_state: Fz.Converter;
export declare const livolo_dimmer_state: Fz.Converter;
export declare const livolo_cover_state: Fz.Converter;
export declare const livolo_hygrometer_state: Fz.Converter;
export declare const livolo_illuminance_state: Fz.Converter;
export declare const livolo_pir_state: Fz.Converter;
export declare const easycode_action: Fz.Converter;
export declare const easycodetouch_action: Fz.Converter;
export declare const livolo_switch_state_raw: Fz.Converter;
export declare const ptvo_switch_uart: Fz.Converter;
export declare const ptvo_switch_analog_input: Fz.Converter;
export declare const keypad20states: Fz.Converter;
export declare const keypad20_battery: Fz.Converter;
export declare const plaid_battery: Fz.Converter;
export declare const heiman_ir_remote: Fz.Converter;
export declare const meazon_meter: Fz.Converter;
export declare const danfoss_thermostat: Fz.Converter;
export declare const danfoss_thermostat_setpoint_scheduled: Fz.Converter;
export declare const danfoss_icon_floor_sensor: Fz.Converter;
export declare const danfoss_icon_battery: Fz.Converter;
export declare const danfoss_icon_regulator: Fz.Converter;
export declare const danfoss_icon_hvac_user_interface: Fz.Converter;
export declare const orvibo_raw_1: Fz.Converter;
export declare const orvibo_raw_2: Fz.Converter;
export declare const tint_scene: Fz.Converter;
export declare const tint404011_move_to_color_temp: Fz.Converter;
export declare const restorable_brightness: Fz.Converter;
export declare const ewelink_action: Fz.Converter;
export declare const diyruz_contact: Fz.Converter;
export declare const diyruz_rspm: Fz.Converter;
export declare const K4003C_binary_input: Fz.Converter;
export declare const enocean_ptm215z: Fz.Converter;
export declare const enocean_ptm215ze: Fz.Converter;
export declare const enocean_ptm216z: Fz.Converter;
export declare const _8840100H_water_leak_alarm: Fz.Converter;
export declare const diyruz_freepad_clicks: Fz.Converter;
export declare const kmpcil_res005_occupancy: Fz.Converter;
export declare const kmpcil_res005_on_off: Fz.Converter;
export declare const _3310_humidity: Fz.Converter;
export declare const smartthings_acceleration: Fz.Converter;
export declare const byun_smoke_false: Fz.Converter;
export declare const byun_smoke_true: Fz.Converter;
export declare const byun_gas_false: Fz.Converter;
export declare const byun_gas_true: Fz.Converter;
export declare const hue_smart_button_event: Fz.Converter;
export declare const legrand_binary_input_moving: Fz.Converter;
export declare const legrand_binary_input_on_off: Fz.Converter;
export declare const bticino_4027C_binary_input_moving: Fz.Converter;
export declare const legrand_scenes: Fz.Converter;
export declare const legrand_master_switch_center: Fz.Converter;
export declare const legrand_pilot_wire_mode: Fz.Converter;
export declare const legrand_power_alarm: Fz.Converter;
export declare const legrand_greenpower: Fz.Converter;
export declare const W2_module_carbon_monoxide: Fz.Converter;
export declare const command_status_change_notification_action: Fz.Converter;
export declare const ptvo_multistate_action: Fz.Converter;
export declare const konke_action: Fz.Converter;
export declare const qlwz_letv8key_switch: Fz.Converter;
export declare const keen_home_smart_vent_pressure: Fz.Converter;
export declare const U02I007C01_contact: Fz.Converter;
export declare const U02I007C01_water_leak: Fz.Converter;
export declare const heiman_hcho: Fz.Converter;
export declare const heiman_air_quality: Fz.Converter;
export declare const scenes_recall_scene_65024: Fz.Converter;
export declare const adeo_button_65024: Fz.Converter;
export declare const color_stop_raw: Fz.Converter;
export declare const almond_click: Fz.Converter;
export declare const SAGE206612_state: Fz.Converter;
export declare const ZMCSW032D_cover_position: Fz.Converter;
export declare const PGC410EU_presence: Fz.Converter;
export declare const STS_PRS_251_presence: Fz.Converter;
export declare const heiman_scenes: Fz.Converter;
export declare const javis_lock_report: Fz.Converter;
export declare const diyruz_freepad_config: Fz.Converter;
export declare const diyruz_geiger: Fz.Converter;
export declare const diyruz_geiger_config: Fz.Converter;
export declare const diyruz_airsense_config_co2: Fz.Converter;
export declare const diyruz_airsense_config_temp: Fz.Converter;
export declare const diyruz_airsense_config_pres: Fz.Converter;
export declare const diyruz_airsense_config_hum: Fz.Converter;
export declare const diyruz_zintercom_config: Fz.Converter;
export declare const CC2530ROUTER_led: Fz.Converter;
export declare const CC2530ROUTER_meta: Fz.Converter;
export declare const KAMI_contact: Fz.Converter;
export declare const KAMI_occupancy: Fz.Converter;
export declare const DNCKAT_S00X_buttons: Fz.Converter;
export declare const hue_motion_sensitivity: Fz.Converter;
export declare const hue_motion_led_indication: Fz.Converter;
export declare const hue_wall_switch_device_mode: Fz.Converter;
export declare const CCTSwitch_D0001_levelctrl: Fz.Converter;
export declare const CCTSwitch_D0001_lighting: Fz.Converter;
export declare const hue_wall_switch: Fz.Converter;
export declare const hue_dimmer_switch: Fz.Converter;
export declare const hue_tap: Fz.Converter;
export declare const hue_twilight: Fz.Converter;
export declare const tuya_relay_din_led_indicator: Fz.Converter;
export declare const ias_keypad: Fz.Converter;
export declare const itcmdr_clicks: Fz.Converter;
export declare const ZB003X_attr: Fz.Converter;
export declare const ZB003X_occupancy: Fz.Converter;
export declare const idlock: Fz.Converter;
export declare const idlock_fw: Fz.Converter;
export declare const schneider_pilot_mode: Fz.Converter;
export declare const schneider_ui_action: Fz.Converter;
export declare const schneider_temperature: Fz.Converter;
export declare const wiser_smart_thermostat_client: Fz.Converter;
export declare const wiser_smart_setpoint_command_client: Fz.Converter;
export declare const rc_110_level_to_scene: Fz.Converter;
export declare const heiman_doorbell_button: Fz.Converter;
export declare const sihas_people_cnt: Fz.Converter;
export declare const sihas_action: Fz.Converter;
export declare const tuya_operation_mode: Fz.Converter;
export declare const sunricher_switch2801K2: Fz.Converter;
export declare const sunricher_switch2801K4: Fz.Converter;
export declare const command_stop_move_raw: Fz.Converter;
export declare const tuya_multi_action: Fz.Converter;
export declare const led_on_motion: Fz.Converter;
export declare const hw_version: Fz.Converter;
export declare const SNZB02_temperature: Fz.Converter;
export declare const SNZB02_humidity: Fz.Converter;
export declare const awox_colors: Fz.Converter;
export declare const awox_refreshColored: Fz.Converter;
export declare const awox_refresh: Fz.Converter;
export declare const ignore_onoff_report: Fz.Converter;
export declare const ignore_basic_report: Fz.Converter;
export declare const ignore_illuminance_report: Fz.Converter;
export declare const ignore_occupancy_report: Fz.Converter;
export declare const ignore_temperature_report: Fz.Converter;
export declare const ignore_humidity_report: Fz.Converter;
export declare const ignore_pressure_report: Fz.Converter;
export declare const ignore_analog_report: Fz.Converter;
export declare const ignore_multistate_report: Fz.Converter;
export declare const ignore_power_report: Fz.Converter;
export declare const ignore_light_brightness_report: Fz.Converter;
export declare const ignore_light_color_colortemp_report: Fz.Converter;
export declare const ignore_closuresWindowCovering_report: Fz.Converter;
export declare const ignore_thermostat_report: Fz.Converter;
export declare const ignore_iaszone_attreport: Fz.Converter;
export declare const ignore_iaszone_statuschange: Fz.Converter;
export declare const ignore_iaszone_report: Fz.Converter;
export declare const ignore_iasace_commandgetpanelstatus: Fz.Converter;
export declare const ignore_genIdentify: Fz.Converter;
export declare const ignore_command_on: Fz.Converter;
export declare const ignore_command_off: Fz.Converter;
export declare const ignore_command_off_with_effect: Fz.Converter;
export declare const ignore_command_step: Fz.Converter;
export declare const ignore_command_stop: Fz.Converter;
export declare const ignore_poll_ctrl: Fz.Converter;
export declare const ignore_genLevelCtrl_report: Fz.Converter;
export declare const ignore_genOta: Fz.Converter;
export declare const ignore_haDiagnostic: Fz.Converter;
export declare const ignore_zclversion_read: Fz.Converter;
export declare const ignore_time_read: Fz.Converter;
export declare const ignore_tuya_set_time: Fz.Converter;
export declare const ignore_tuya_raw: Fz.Converter;
export declare const ignore_metering: Fz.Converter;
export declare const ignore_electrical_measurement: Fz.Converter;
export declare const command_arm_with_transaction: Fz.Converter;
export declare const metering_datek: Fz.Converter;
export declare const EKO09738_metering: Fz.Converter;
export declare const command_on_presence: Fz.Converter;
export declare const ias_ace_occupancy_with_timeout: Fz.Converter;
export declare const SP600_power: Fz.Converter;
export declare const stelpro_thermostat: Fz.Converter;
export declare const viessmann_thermostat: Fz.Converter;
export declare const eurotronic_thermostat: Fz.Converter;
export declare const terncy_raw: Fz.Converter;
export declare const ZM35HQ_attr: Fz.Converter;
export declare const schneider_lighting_ballast_configuration: Fz.Converter;
export declare const wiser_lighting_ballast_configuration: Fz.Converter;
export declare const wiser_smart_thermostat: Fz.Converter;
export declare const TS110E: Fz.Converter;
export declare const TS110E_light_type: Fz.Converter;
export declare const TS110E_switch_type: Fz.Converter;
//# sourceMappingURL=fromZigbee.d.ts.map