import{u as i,j as s}from"./index-a77bd902.js";const c=t=>{const{description:e}=t,{t:r}=i("settingsSchemaDescriptions");return e?s.jsx("div",{children:s.jsx("div",{className:"mb-3",children:r(e)})}):s.jsx(s.Fragment,{})},a=({title:t})=>{const{t:e}=i("settingsSchemaTitles");return s.jsxs("div",{className:"my-1",children:[s.jsx("h5",{children:e(t)}),s.jsx("hr",{className:"border-0 bg-secondary",style:{height:"1px"}})]})};export{c as D,a as T};
