"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.definitions = void 0;
const modernExtend_1 = require("../lib/modernExtend");
exports.definitions = [
    {
        zigbeeModel: ["SM0502"],
        model: "SM0502",
        vendor: "SIMON",
        description: "i7 2-gang smart dimming switch",
        extend: [(0, modernExtend_1.deviceEndpoints)({ endpoints: { left: 1, right: 2 } }), (0, modernExtend_1.light)({ endpointNames: ["left", "right"] })],
    },
];
//# sourceMappingURL=simon.js.map