import { EventEmitter } from 'node:events';
export declare const frameParserEvents: EventEmitter<[never]>;
declare function processFrame(frame: Uint8Array): void;
export default processFrame;
//# sourceMappingURL=frameParser.d.ts.map