import * as m from "../lib/modernExtend";
import type { DefinitionWithExtend, ModernExtend } from "../lib/types";
export declare function sengledLight(args?: m.LightArgs): ModernExtend;
export declare function sengledSwitchAction(): ModernExtend;
export declare const definitions: DefinitionWithExtend[];
//# sourceMappingURL=sengled.d.ts.map